@@ -561,6 +561,15 @@ void stack::call::relay(thread *thread, session *s)
 	osip_message_get_body(thread->sevent->response, 0, &body);
 
 	switch(s->state) {
+	case session::REFER:
+		if(status == SIP_ACCEPTED) {
+			set(TRANSFER, 'x', "transfer");
+			disconnectLocked();
+			Mutex::release(this);
+			return;
+		}
+		s->state = session::OPEN;
+		break;
 	case session::REINVITE:
 		if(status != SIP_ACCEPTED)
 			s->state = session::OPEN;
@@ -826,6 +835,10 @@ void stack::call::log(void)
 	String::set(node->dialed, sizeof(node->dialed), dialed);
 	String::set(node->joined, sizeof(node->joined), joined);
 	String::set(node->display, sizeof(node->display), source->display);
+	if(target)
+		snprintf(node->network, sizeof(node->network), "%s/%s", source->network, target->network);
+	else
+		String::set(node->network, sizeof(node->network), source->network);
 	cdr::post(node);	
 	starting = 0l;
 }
